from os import X_OK, access
from pathlib import Path

from yaml import FullLoader, dump, load

from recoverpy.utils import errors
from recoverpy.utils.logger import LOGGER
from recoverpy.utils.saver import SAVER

_CONFIG_DIR = Path(__file__).parent.absolute()


def path_is_valid(path: str) -> bool:
    """Verify validity of a given path.

    Args:
        path (str): Path to verify

    Returns:
        bool: Is path valid
    """
    path = Path(path)
    try:
        if not path.is_dir():
            return False
    except PermissionError:
        return False

    return access(path, X_OK)


def write_config(save_path: str, log_path: str, enable_logging: bool):
    """Write provided configuration to config file.

    Args:
        save_path (str): Provided save path
        log_path (str): Provided log path
        enable_logging (bool): Logging is enabled or disabled
    """
    with open(_CONFIG_DIR / "config.yaml", "w") as config_file:
        config = {
            "save_directory": save_path,
            "log_directory": log_path,
            "enable_logging": enable_logging,
        }

        dump(config, config_file)
        load_config()


def load_config():
    """Set logging and saving parameters based on yaml conf file.

    Raises:
        errors.NoSavePath: If config file save path is empty
        errors.InvalidSavePath: If config file save path is invalid
        errors.InvalidLogPath: If config file log path is invalid
    """

    with open(_CONFIG_DIR / "config.yaml", "r") as config_file:
        config = load(config_file, Loader=FullLoader)

    if config["save_directory"] == "":
        raise errors.NoSavePath
    if not path_is_valid(config["save_directory"]):
        raise errors.InvalidSavePath

    SAVER.save_path = config["save_directory"]

    if config["log_directory"] == "":
        LOGGER.log_enabled = False
    elif not path_is_valid(config["log_directory"]):
        raise errors.InvalidLogPath
    else:
        LOGGER.log_path = config["log_directory"]

    LOGGER.log_enabled = bool(config["enable_logging"])
    if LOGGER.log_enabled:
        LOGGER.start_logging()



from logging import getLogger
from os import environ

from recoverpy.config.config import load_config
from recoverpy.utils.helper import is_installed
from recoverpy.views_handler import VIEWS_HANDLER

getLogger(__name__)


def verify_terminal_conf():
    """Fix for older terminals."""
    term = environ["TERM"]

    if term != "xterm-256color":
        environ["TERM"] = "xterm-256color"


def verify_dependencies():
    """Verify the availability of mandatory dependencies.

    Raises:
        OSError: If a dependency is missing.
    """
    for dep in ["grep", "dd", "lsblk"]:
        if not is_installed(command=dep):
            raise OSError(
                f"Command '{dep}' is unavailable on your system.\n"
                "Please verify your configuration."
            )


def main():
    """Set configuration and start UI."""
    verify_dependencies()
    verify_terminal_conf()
    load_config()

    VIEWS_HANDLER.open_view_parameters()



from py_cui import PyCUI

from recoverpy import views_handler as views_handler
from recoverpy.utils.logger import LOGGER as LOGGER
from recoverpy.utils.saver import SAVER as SAVER
from recoverpy.views.menu_with_block_display import MenuWithBlockDisplay


class ResultsView(MenuWithBlockDisplay):
    """ResultsView let the user navigate through partition blocks and save results.

    Args:
        _BLOCK_DISPLAY_MENU (MenuWithBlockDisplay): Composition to inherit block display
        methods.

    Attributes:
        saved_blocks_dict (dict): Used to store block numbers associated with their text
            contents. Will be used to save it in a file.
        current_block (int): Partition block number currently displayed.
    """

    def __init__(self, master: PyCUI, partition: str, initial_block: int):
        """Initialize ResultsView.

        Args:
            master (PyCUI): PyCUI main object for UI.
            partition (str): System partition to search.
            initial_block (int): Initial partition block number that will be displayed.
        """
        super().__init__()

        self.master = master
        self.partition = partition

        LOGGER.write("info", "Starting 'ResultsView' CUI window")

        self.saved_blocks_dict = {}

        self.current_block = initial_block

        self.create_ui_content()
        # Display initial block at opening
        self.display_block(self.current_block)

    def create_ui_content(self):
        """Handle the creation of the UI elements."""
        self.previous_button = self.master.add_button(
            "<",
            3,
            0,
            row_span=3,
            column_span=1,
            padx=1,
            pady=0,
            command=self.display_previous_block,
        )
        self.previous_button.set_color(1)

        self.next_button = self.master.add_button(
            ">",
            3,
            9,
            row_span=3,
            column_span=1,
            padx=1,
            pady=0,
            command=self.display_next_block,
        )
        self.next_button.set_color(1)

        self.result_content_box = self.master.add_text_block(
            "Block content:", 0, 1, row_span=9, column_span=8, padx=1, pady=0
        )
        self.result_content_box.set_title(f"Block {self.current_block}")

        self.add_result_button = self.master.add_button(
            "Add current block to file",
            9,
            0,
            row_span=1,
            column_span=5,
            padx=1,
            pady=0,
            command=self.add_block_to_file,
        )
        self.add_result_button.set_color(6)

        self.save_file_button = self.master.add_button(
            "Save file",
            9,
            5,
            row_span=1,
            column_span=3,
            padx=1,
            pady=0,
            command=self.save_file,
        )
        self.save_file_button.set_color(4)

        self.go_back_button = self.master.add_button(
            "Go back to previous view",
            9,
            8,
            row_span=1,
            column_span=2,
            padx=1,
            pady=0,
            command=views_handler.VIEWS_HANDLER.results_go_back,
        )
        self.go_back_button.set_color(2)

    def add_block_to_file(self):
        """Store currently displayed result in a dict to save it later."""
        if self.current_block in self.saved_blocks_dict:
            return

        self.master.show_message_popup("", "Result added to file")
        self.saved_blocks_dict[self.current_block] = self.current_result
        LOGGER.write(
            "debug",
            f"Stored block {self.current_block} for future save",
        )

    def save_file(self):
        """Bundle results saving and popup message."""
        len_results = len(self.saved_blocks_dict)

        if len_results == 0:
            self.master.show_message_popup("", "No result to save yet")
            return

        SAVER.save_result_dict(self.saved_blocks_dict)

        if len_results == 1:
            self.master.show_message_popup(
                "",
                f"Block saved in {SAVER.last_saved_file}",
            )
        else:
            self.master.show_message_popup(
                "",
                f"{len_results} blocks saved in {SAVER.last_saved_file}",
            )



from queue import Queue
from re import findall
from shlex import quote
from time import sleep

from py_cui import BLACK_ON_GREEN, PyCUI, keys

from recoverpy import search, views_handler
from recoverpy.utils.logger import LOGGER
from recoverpy.utils.saver import SAVER
from recoverpy.views.menu_with_block_display import MenuWithBlockDisplay


class SearchView(MenuWithBlockDisplay):
    """SearchView displays search results and corresponding block contents.

    Args:
        _BLOCK_DISPLAY_MENU (MenuWithBlockDisplay): Composition to inherit block display
        methods

    Attributes:
        master (PyCUI): PyCUI main object for UI
        queue_object (Queue): Queue object where grep command stdout will be stored
        result_index (int): Number of results already processed
        grep_progress (str): Formated output of 'progress' command
        partition (str): System partition selected by user for search
        block_size (int): Size of partition block for dd parsing
        searched_string (str): String given by the user that will be searched by dd
    """

    def __init__(self, master: PyCUI, partition: str, string_to_search: str):
        """Initialize SearchView.

        Args:
            master (PyCUI): PyCUI main object for UI
            partition (str): System partition to search
            string_to_search (str): String to search in partition blocks
        """
        super().__init__()

        self.master = master
        self.master.set_refresh_timeout(1)

        self.queue_object = Queue()
        self.result_index = 0

        self.grep_progress = ""
        self.inodes = []
        self.partition = partition
        self.block_size = 512

        self.searched_string = string_to_search

        LOGGER.write("info", "Starting 'SearchView' CUI window")

        self.create_ui_content()

        search.start_search(self)

        LOGGER.write(
            "info",
            f"Raw searched string:\n{self.searched_string}",
        )
        LOGGER.write(
            "info",
            f"Formated searched string:\n{quote(self.searched_string)}",
        )

    def set_title(self):
        """Set window title based on number of results and search progress."""
        if self.grep_progress != "":
            title = f"{self.grep_progress} - {self.result_index} results"
        else:
            title = f"{self.result_index} results"

        self.master.set_title(title)

    def create_ui_content(self):
        """Handle the creation of the UI elements."""
        self.search_results_scroll_menu = self.master.add_scroll_menu(
            "Search results:", 0, 0, row_span=10, column_span=5, padx=1, pady=0
        )
        self.search_results_scroll_menu.add_text_color_rule(
            self.searched_string,
            BLACK_ON_GREEN,
            "contains",
            match_type="regex",
        )
        self.search_results_scroll_menu.add_key_command(
            keys.KEY_ENTER,
            self.display_selected_block,
        )

        self.result_content_box = self.master.add_text_block(
            "Block content:", 0, 5, row_span=9, column_span=5, padx=1, pady=0
        )
        self.result_content_box.add_key_command(
            keys.KEY_F5,
            self.open_save_popup,
        )
        self.result_content_box.add_key_command(
            keys.KEY_F6,
            self.display_previous_block,
        )
        self.result_content_box.add_key_command(
            keys.KEY_F7,
            self.display_next_block,
        )

        self.previous_button = self.master.add_button(
            "<",
            9,
            5,
            row_span=1,
            column_span=1,
            padx=1,
            pady=0,
            command=self.display_previous_block,
        )
        self.previous_button.set_color(1)

        self.next_button = self.master.add_button(
            ">",
            9,
            8,
            row_span=1,
            column_span=1,
            padx=1,
            pady=0,
            command=self.display_next_block,
        )
        self.next_button.set_color(1)

        self.save_file_button = self.master.add_button(
            "Save Block",
            9,
            6,
            row_span=1,
            column_span=2,
            padx=1,
            pady=0,
            command=self.open_save_popup,
        )
        self.save_file_button.set_color(4)

        self.exit_button = self.master.add_button(
            "Exit",
            9,
            9,
            row_span=1,
            column_span=1,
            padx=1,
            pady=0,
            command=self.master.stop,
        )
        self.exit_button.set_color(3)

    def populate_result_list(self):
        """Poll grep output and populate result list."""
        while True:
            try:
                new_results, self.result_index = search.yield_new_results(
                    self.queue_object,
                    self.result_index,
                )
            except TypeError:
                # If no new results
                sleep(1)
                continue

            self.add_results_to_list(new_results=new_results)
            self.set_title()

            # Sleeps to avoid unnecessary overload
            sleep(1)

    def add_results_to_list(self, new_results: list):
        """Add new results from the grep command to the left hand result list.

        Args:
            new_results (list): New results from the grep command.
        """
        for result in new_results:
            string_result = str(result)[2:-1]
            inode = findall(r"^([0-9]+)\:", string_result)[0]
            content = string_result[len(inode) + 1 :]
            self.inodes.append(int(inode))
            self.search_results_scroll_menu.add_item(content)

    def update_block_number(self):
        """Update currently viewed block number upon user selection."""
        inode = self.inodes[
            int(self.search_results_scroll_menu.get_selected_item_index())
        ]
        self.current_block = str(int(inode / self.block_size))
        LOGGER.write("debug", f"Displayed block set to {self.current_block}")

    def display_selected_block(self):
        """Bundle updating + displaying block.

        Called when the user select a result in the left hand list.
        """
        self.update_block_number()
        self.display_block(self.current_block)

    def open_save_popup(self):
        """Open a popup displaying save options."""
        if self.current_block is None:
            self.master.show_message_popup(
                "",
                "Please select a block first.",
            )
            return

        view_choices = [
            "Save currently displayed block",
            "Explore neighboring blocks and save it all",
            "Cancel",
        ]
        self.master.show_menu_popup(
            "How do you want to save it ?",
            view_choices,
            self.handle_save_popup_choice,
        )

    def handle_save_popup_choice(self, choice: str):
        """Launch the action selected by the user (save, explore, exit).

        Args:
            choice (str): User choice given by open_save_popup function.
        """
        if choice == "Save currently displayed block":
            SAVER.save_result(
                current_block=self.current_block,
                result=self.current_result,
            )
            self.master.show_message_popup("", "Result saved.")
        elif choice == "Explore neighboring blocks and save it all":
            views_handler.VIEWS_HANDLER.open_view_results(
                partition=self.partition,
                block=self.current_block,
            )



from datetime import datetime

from recoverpy.utils.logger import LOGGER


class Saver:
    """Encapsulates all result saving related methods.

    Attributes:
        _save_path (str): Local path for search results saving.
        last_saved_file (str): Last saved file to inform user.
    """

    def __init__(self):
        """Initialize Saver."""
        self.save_path = None
        self.last_saved_file = None

    def save_result(self, current_block: str, result: str):
        """Save a single result in a text file.

        Args:
            current_block (str): Current partition block for file naming
            result (str): Block content to be saved
        """
        time_format = datetime.now().strftime("recoverpy-save-%Y-%m-%d-%H%M%S")
        file_name = f"{self.save_path}{time_format}-{current_block}"

        self.write_to_file(file_name=file_name, content=result)

    def save_result_dict(self, results: dict):
        """Order a results dictionnary by block numbers and then save it in a text file.

        Args:
            results (dict): Blocks number and content to be ordered
        """
        ordered_blocks = sorted(results)

        final_output = ""

        for key in ordered_blocks:
            final_output += results[key]
            final_output += "\n"

        date_time_name = datetime.now().strftime("recoverpy-save-%Y-%m-%d-%H%M%S")
        file_name = f"{self.save_path}{date_time_name}"

        self.write_to_file(file_name=file_name, content=final_output)

    def write_to_file(self, file_name: str, content: str):
        """Write content provided save file.

        Args:
            file_name (str): Save file name
            content (str): File content to be written
        """
        with open(file_name, "w") as save_file:
            save_file.write(content)

        self.last_saved_file = file_name

        LOGGER.write("info", f"Output saved in file {file_name}")


SAVER = Saver()



import logging
from datetime import datetime


class Logger:
    """Encapsulates all logging related methods.

    Attributes:
        _log_path (str): Local path for search results saving.
        _log_enabled (bool): Last saved file to inform user.
        _logger (logging.Logger): Wrapped Logger object.
    """

    def __init__(self):
        """Initialize Logger."""
        self.log_path = None
        self.log_enabled = False
        self._logger = None

    def start_logging(self):
        """Initiate and configure the logger object."""
        time = datetime.now().strftime("%Y-%m-%d-%H%M%S")

        log_file_name = f"{self.log_path}recoverpy-{time}.log"

        self._logger = logging.getLogger("main")
        self._logger.setLevel(logging.DEBUG)

        file_handler = logging.FileHandler(log_file_name)
        file_handler.setLevel(logging.DEBUG)

        self._logger.addHandler(file_handler)

    def write(self, log_type: str, text: str):
        """Use logger object to write to file.

        Args:
            log_type (str): Log level
            text (str): Message
        """
        if not self.log_enabled:
            return

        if log_type == "debug":
            self._logger.debug(text)
        elif log_type == "info":
            self._logger.info(text)
        elif log_type == "warning":
            self._logger.warning(text)
        elif log_type == "error":
            self._logger.error(text)
        elif log_type == "critical":
            self._logger.critical(text)


LOGGER = Logger()



from re import findall

from py_cui import GREEN_ON_BLACK, YELLOW_ON_BLACK, PyCUI, keys

from recoverpy import views_handler
from recoverpy.utils import helper
from recoverpy.utils.logger import LOGGER


class ParametersView:
    """ParametersView prompts to select a partion and a string to search.

    User is prompted to select a partition and a string to search in it.

    Attributes:
        partition_to_search (str): Partition selected by user.
        string_to_search (str): String entered by user.
        partitions_dict (dict): Dictionnary of system partitions found with
            lsblk command and their attributes.
    """

    def __init__(self, master: PyCUI):
        """Initialize ParametersView.

        Args:
            master (PyCUI): PyCUI main object for UI
        """
        self.master = master

        self.partition_to_search = None
        self.string_to_search = None
        self.partitions_dict = None

        LOGGER.write("info", "Starting 'ParametersView' CUI window")
        helper.is_user_root(window=self.master)

        self.create_ui_content()
        self.get_system_partitions()
        self.add_partitions_to_list()

    def create_ui_content(self):
        """Handle the creation of the UI elements."""
        self.partitions_list_scroll_menu = self.master.add_scroll_menu(
            "Select a partition to search:", 0, 0, row_span=9, column_span=5
        )
        self.partitions_list_scroll_menu.add_key_command(
            keys.KEY_ENTER, self.select_partition
        )

        # Color rules
        self.partitions_list_scroll_menu.add_text_color_rule(
            "Mounted at",
            YELLOW_ON_BLACK,
            "contains",
        )
        self.partitions_list_scroll_menu.set_selected_color(GREEN_ON_BLACK)

        self.string_text_box = self.master.add_text_block(
            "Enter a text to search:",
            0,
            5,
            row_span=9,
            column_span=5,
        )

        self.confirm_search_button = self.master.add_button(
            "Start",
            9,
            4,
            row_span=1,
            column_span=2,
            padx=0,
            pady=0,
            command=self.confirm_search,
        )
        self.confirm_search_button.set_color(4)

        self.open_config_button = self.master.add_button(
            "Settings",
            9,
            8,
            row_span=1,
            column_span=2,
            padx=1,
            pady=0,
            command=views_handler.VIEWS_HANDLER.open_view_config,
        )
        self.open_config_button.set_color(1)

    def get_system_partitions(self):
        """Call lsblk and lsblk output formatting."""
        partitions_list = helper.lsblk()
        self.partitions_dict = helper.format_partitions_list(
            window=self.master,
            raw_lsblk=partitions_list,
        )

    def add_partitions_to_list(self):
        """Populate the partition list with lsblk output."""
        if self.partitions_dict is None:
            return

        for partition in self.partitions_dict:
            if self.partitions_dict[partition]["IS_MOUNTED"]:
                self.partitions_list_scroll_menu.add_item(
                    f"Name: {partition}  -  "
                    f"Type: {self.partitions_dict[partition]['FSTYPE']}  -  "
                    f"Mounted at: {self.partitions_dict[partition]['MOUNT_POINT']}"
                )
            else:
                self.partitions_list_scroll_menu.add_item(
                    f"Name: {partition}  -  "
                    f"Type: {self.partitions_dict[partition]['FSTYPE']}"
                )

            LOGGER.write("debug", f"Partition added to list: {partition}")

    def select_partition(self):
        """Handle the user selection of a partition in the list."""
        selected_partition = findall(
            r"Name\:\ ([^\ \n]+)\ ",
            self.partitions_list_scroll_menu.get(),
        )[0]

        if self.partitions_dict[selected_partition]["IS_MOUNTED"]:
            # Warn the user to unmount his partition first
            self.master.show_warning_popup(
                "You probably should unmount first !",
                f"It is highly recommended to unmount {selected_partition}"
                " ASAP to avoid any data loss.",
            )
        else:
            self.master.show_message_popup(
                "",
                f"Partition {selected_partition} selected.",
            )

        self.partition_to_search = "/dev/" + selected_partition.strip()

        LOGGER.write(
            "info",
            f"Partition selected: {self.partition_to_search}",
        )

    def confirm_search(self):
        """Check if partition is selected and string is given.
        If all required elements are present, launch start_search method.
        """
        if not helper.is_user_root(window=self.master):
            return

        self.string_to_search = self.string_text_box.get()

        LOGGER.write("info", "Starting search")

        if self.partition_to_search == "":
            # No partition selected
            self.master.show_message_popup(
                "Whoops !",
                "You have to select a partition to search.",
            )
            LOGGER.write("warning", "No partition selected for search")
        elif not self.string_to_search.strip():
            # Blank string to search
            self.master.show_message_popup(
                "Oops !",
                "You have to enter a text to search.",
            )
            LOGGER.write("warning", "No string given for search")
        else:
            # Prompt to confirm string
            self.master.show_yes_no_popup(
                "Do you want to search this text on partition "
                f"{self.partition_to_search} ?",
                self.start_search,
            )

    def start_search(self, is_confirmed: bool):
        """Close parameters view and open search view if confirmed.

        Args:
            is_confirmed (bool): User popup selection
        """
        if is_confirmed:
            views_handler.VIEWS_HANDLER.close_view_parameters()
            views_handler.VIEWS_HANDLER.open_view_search(
                partition=self.partition_to_search,
                string_to_search=self.string_to_search.strip(),
            )



from py_cui import PyCUI

from recoverpy import views_handler as views_handler
from recoverpy.config import config as CONFIG
from recoverpy.utils.logger import LOGGER
from recoverpy.utils.saver import SAVER


class ConfigView:
    """ConfigView is used to display and edit configuration.

    Attributes:
        _log_enabled (bool): Mirror LOGGER state for button colorization
    """

    def __init__(self, master: PyCUI):
        """Initialize ConfigView.

        Args:
            master (PyCUI): PyCUI main object for UI.
        """
        self.master = master
        self._log_enabled = LOGGER.log_enabled
        self.create_ui_content()

    def create_ui_content(self):
        """Handle the creation of the UI elements."""
        self.save_path_box = self.master.add_text_box(
            title="Save Path",
            row=0,
            column=1,
            row_span=1,
            column_span=8,
            padx=0,
            pady=0,
            initial_text=SAVER.save_path,
        )
        self.master.add_button(
            "Save",
            row=1,
            column=8,
            row_span=1,
            column_span=1,
            padx=0,
            pady=0,
            command=self.set_save_path,
        ).set_color(1)
        self.log_path_box = self.master.add_text_box(
            title="Log Path",
            row=2,
            column=1,
            row_span=1,
            column_span=8,
            padx=0,
            pady=0,
            initial_text=LOGGER.log_path,
        )
        self.master.add_button(
            "Save",
            row=3,
            column=8,
            row_span=1,
            column_span=1,
            padx=0,
            pady=0,
            command=self.set_log_path,
        ).set_color(1)
        self.master.add_label(
            title="Enable Logging",
            row=4,
            column=4,
            row_span=1,
            column_span=2,
            padx=0,
            pady=0,
        ).selectable = False
        self.yes_button = self.master.add_button(
            "Yes",
            row=5,
            column=3,
            row_span=1,
            column_span=1,
            padx=0,
            pady=0,
            command=self.enable_logging,
        )
        self.no_button = self.master.add_button(
            "No",
            row=5,
            column=6,
            row_span=1,
            column_span=1,
            padx=0,
            pady=0,
            command=self.disable_logging,
        )
        self.master.add_button(
            "Save & Exit",
            row=8,
            column=2,
            row_span=1,
            column_span=2,
            padx=0,
            pady=0,
            command=self.save_all,
        ).set_color(4)
        self.master.add_button(
            "Cancel",
            row=8,
            column=6,
            row_span=1,
            column_span=2,
            padx=0,
            pady=0,
            command=views_handler.VIEWS_HANDLER.config_go_back,
        ).set_color(2)

        self.set_yes_no_colors()

    def set_save_path(self):
        """Verify and set provided save path."""
        user_input = self.save_path_box.get()
        if not CONFIG.path_is_valid(path=user_input):
            self.master.show_error_popup("Path invalid", "Given save path is invalid.")
            return

        CONFIG.write_config(
            save_path=user_input,
            log_path=LOGGER.log_path,
            enable_logging=LOGGER.log_enabled,
        )

        self.master.show_message_popup("", "Save path changed successfully")

    def set_log_path(self):
        """Verify and set provided log path."""
        user_input = self.log_path_box.get()
        if not CONFIG.path_is_valid(path=user_input):
            self.master.show_error_popup("Path invalid", "Given log path is invalid.")
            return

        CONFIG.write_config(
            save_path=SAVER.save_path,
            log_path=user_input,
            enable_logging=LOGGER.log_enabled,
        )

        self.master.show_message_popup("", "Log path changed successfully")

    def enable_logging(self):
        """Enable logging on button press."""
        if self._log_enabled:
            return

        self.set_log_state(enabled=True)
        self.master.show_message_popup("", "Logging enabled")

    def disable_logging(self):
        """Disable logging on button press."""
        if not self._log_enabled:
            return

        self.set_log_state(enabled=False)
        self.master.show_message_popup("", "Logging disabled")

    def set_log_state(self, enabled: bool):
        """Set disared log state.

        Args:
            enabled (bool): Logging enabled or disabled
        """
        self._log_enabled = enabled
        CONFIG.write_config(
            save_path=SAVER.save_path,
            log_path=LOGGER.log_path,
            enable_logging=self._log_enabled,
        )
        self.set_yes_no_colors()

    def set_yes_no_colors(self):
        """Colorize log state buttons."""
        if self._log_enabled:
            self.yes_button.set_color(4)
            self.no_button.set_color(1)
        else:
            self.yes_button.set_color(1)
            self.no_button.set_color(4)

    def save_all(self):
        """Save full config and go back to first menu."""
        save_path = self.save_path_box.get()
        log_path = self.log_path_box.get()
        if not CONFIG.path_is_valid(path=save_path):
            self.master.show_error_popup("Path invalid", "Given save path is invalid.")
            return
        if not CONFIG.path_is_valid(path=log_path):
            self.master.show_error_popup("Path invalid", "Given Log path is invalid.")
            return

        CONFIG.write_config(
            save_path=save_path,
            log_path=log_path,
            enable_logging=self._log_enabled,
        )
        views_handler.VIEWS_HANDLER.config_go_back()



from py_cui import PyCUI

from recoverpy.views import view_config, view_parameters, view_results, view_search


class ViewsHandler:
    """Store UI windows instances and provide navigation logic.

    Attributes:
        _parameters_view_window (PyCUI): Parameters window.
        _config_view_window  (PyCUI): Config window.
        _search_view_window (PyCUI): Search window.
        _results_view_window (PyCUI): Results window.
    """

    def __init__(self):
        """Initialize ViewsHandler."""
        self._parameters_view_window = None
        self._config_view_window = None
        self._search_view_window = None
        self._results_view_window = None

    def create_view(self):
        """Create a PyCUI instance with standard attributes.

        Returns:
            PyCUI: Created view
        """
        view = PyCUI(10, 10)
        view.toggle_unicode_borders()
        view.set_title("RecoverPy 1.4.1")

        return view

    def open_view_parameters(self):
        """Start a ParametersView instance."""
        self._parameters_view_window = self.create_view()

        view_parameters.ParametersView(self._parameters_view_window)
        self._parameters_view_window.start()

    def close_view_parameters(self):
        """Stop the ParametersView instance."""
        if self._parameters_view_window is None:
            return
        self._parameters_view_window.stop()

    def open_view_config(self):
        """Start a ConfigView instance."""
        self.close_view_parameters()
        self._config_view_window = self.create_view()

        view_config.ConfigView(self._config_view_window)
        self._config_view_window.start()

    def close_view_config(self):
        """Stop the ParametersView instance."""
        if self._config_view_window is None:
            return
        self._config_view_window.stop()

    def open_view_search(self, partition: str, string_to_search: str):
        """Start a SearchView instance.

        Args:
            partition (str): Selected system partition
            string_to_search (str): String to be searched
        """
        self.close_view_results()
        self._search_view_window = self.create_view()

        view_search.SearchView(
            self._search_view_window,
            partition=partition,
            string_to_search=string_to_search,
        )
        self._search_view_window.start()

    def close_view_search(self):
        """Stop the SearchView instance."""
        if self._search_view_window is None:
            return
        self._search_view_window.stop()

    def open_view_results(self, partition: str, block: str):
        """Start a ResultsView instance.

        Args:
            partition (str): Selected system partition
            block (str): Block to be displayed
        """
        self.close_view_search()
        self._results_view_window = self.create_view()
        view_results.ResultsView(
            self._results_view_window,
            partition=partition,
            initial_block=block,
        )
        self._results_view_window.start()

    def close_view_results(self):
        """Stop the ResultsView instance."""
        if self._results_view_window is None:
            return
        self._results_view_window.stop()

    def config_go_back(self):
        """Go back from config view to parameters view."""
        self.close_view_config()
        self._parameters_view_window._stopped = False
        self._parameters_view_window.start()

    def results_go_back(self):
        """Go back from results view to search view."""
        self.close_view_results()
        self._search_view_window._stopped = False
        self._search_view_window.start()


VIEWS_HANDLER = ViewsHandler()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['recoverpy', 'recoverpy.config', 'recoverpy.utils', 'recoverpy.views']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0', 'py-cui>=0.1.4,<0.2.0']

setup_kwargs = {
    'name': 'recoverpy',
    'version': '1.4.1',
    'description': 'A TUI to recover overwritten or deleted data.',
    'long_description': '<div align="center">\n    <img src="docs/assets/logo.png" alt="RecoverPy">\n</div>\n\n<p align="center">\n    <em>Recover overwritten or deleted data.</em>\n</p>\n\n<p align="center">\n<a href="https://img.shields.io/github/v/release/pablolec/recoverpy" target="_blank">\n    <img src="https://img.shields.io/github/v/release/pablolec/recoverpy" alt="Release">\n</a>\n<a href="https://github.com/PabloLec/recoverpy/blob/main/LICENSE" target="_blank">\n    <img src="https://img.shields.io/github/license/pablolec/recoverpy" alt="License">\n</a>\n<a href="https://pepy.tech/project/recoverpy" target="_blank">\n    <img src="https://static.pepy.tech/personalized-badge/recoverpy?period=total&units=abbreviation&left_color=grey&right_color=red&left_text=downloads" alt="Downloads">\n</a>\n<a href="#" target="_blank">\n    <img src="https://github.com/PabloLec/recoverpy/actions/workflows/recoverpy-tests.yml/badge.svg?branch=main" alt="Tests">\n</a>\n</p>\n\n---\n\n<!--ts-->\n   * [Demo](#Demo)\n   * [Installation](#Installation)\n      * [Dependancies](#arrow_right-dependancies)\n      * [Installation from pip](#arrow_right-installation-from-pip)\n   * [Usage](#Usage)\n   * [Tips](#Tips)\n   * [Contributing](#Contributing)\n<!--te-->\n\n---\n\n# RecoverPy\n\nYou can already find plenty of solutions to recover deleted files, but it can be a hassle to recover overwritten files. RecoverPy searches through every block of your partition to find your request.\n\n## Demo\n\n<p align="center">\n    <img src="docs/assets/demo.gif">\n</p>\n\n## Installation\n\n:penguin: RecoverPy is currently only available on Linux systems.\n\n#### :arrow_right: Dependancies\n\n**Mandatory:** To list and search through your partitions, recoverpy uses `grep`, `dd`, and `lsblk` commands.\n\n**Optional:** To display real time grep progress, you can install `progress`.\n\nTo install all dependencies:\n- Debian-like: `apt install grep coreutils util-linux progress`\n- Arch: `pacman -S grep coreutils util-linux progress`\n- Fedora: `dnf install grep coreutils util-linux progress`\n\n#### :arrow_right: Installation from pip\n\n`python3 -m pip install recoverpy`\n\n## Usage\n\n```bash\npython3 -m recoverpy\n```\n\n:red_circle: **You must have root access to use recoverpy**.\n\n*If you are not logged as root use `sudo recoverpy` or log in with `su -` before execution.*<br/>\n\n---\n\n:one: **Select the system partition** in which your file was. If you are out of luck, you can alternatively search in your home partition, maybe your IDE, text editor, etc. made a backup at some point.\n\n:two: **Type a text string to search**. See tips below for better results.\n\nNote that searching a string in a whole partition may take _a while_. (see [euphemism](https://en.wikipedia.org/wiki/Euphemism "euphemism"))\n\nDefault save path is `/tmp/`, click on Settings to edit configuration.\n\n:three: **Start search**, Results will appear in the left-hand box. \n\n:four: **Select a result** to display the corresponding partition block content in the right-hand box.\n\n:five: Once you have found your precious, **select `Save`**.\n\n:six: You can now either save this block individually or explore neighboring blocks for the remaining parts of the file. You could then save it all in one file.\n\n## Tips\n\n- Always do backups! Yes, maybe too late...\n- **Unmount your partition before you do anything!** Although you can search with your partition still mounted, it is highly recommended to unmount your partition to avoid any alteration to your file.\n\nRegarding the searched string:\n\n- Be concise, find something that could be unique to your file.\n- Stay simple, your string is escaped but exotic characters may affect your results.\n- Try to remember the last edit you have made to your file.\n\nWhen you found your file:\n\n- Use the option to explore neighboring blocks to make sure you do not miss some part of your file.\n\n## Contributing\n\nThank you for considering contributing to RecoverPy.\nAny request, bug report or PR are welcome. Please read the [contributing guide](CONTRIBUTING.md).\n',
    'author': 'PabloLec',
    'author_email': 'pablo.lecolinet@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/PabloLec/recoverpy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)



from os import geteuid
from subprocess import call, check_output

from py_cui import PyCUI

from recoverpy.utils.logger import LOGGER


def is_user_root(window: PyCUI) -> bool:
    """Check if user has root privileges.

    The method is simply verifying if EUID == 0.
    It may be problematic in some edge cases. (Some particular OS)
    But, as grep search will not exit quickly, exception handling
    can't be used.

    Args:
        window (PyCUI): PyCUI window to display popup.

    Returns:
        bool: User is root
    """
    if geteuid() == 0:
        LOGGER.write("info", "User is root")
        return True

    window.show_error_popup("Not root :(", "You have to be root or use sudo.")
    LOGGER.write("warning", "User is not root")
    return False


def lsblk() -> list:
    """Use lsblk to generate a list of detected system partitions."

    Returns:
        list: List of system partitions.
    """
    lsblk_output = check_output(
        ["lsblk", "-r", "-n", "-o", "NAME,TYPE,FSTYPE,MOUNTPOINT"],
        encoding="utf-8",
    )
    partitions_list_raw = [
        line.strip()
        for line in lsblk_output.splitlines()
        if " loop " not in line and "swap" not in line
    ]
    partitions_list_formatted = [line.split(" ") for line in partitions_list_raw]

    LOGGER.write(
        "debug",
        str(partitions_list_formatted),
    )

    return partitions_list_formatted


def format_partitions_list(window: PyCUI, raw_lsblk: list) -> dict:
    """Format found partition list to a dict.

    Args:
        window (PyCUI): PyCUI window to display popup.
        raw_lsblk (list): Raw lsblk output.

    Returns:
        dict: Found partitions with format :
            {Name: FSTYPE, IS_MOUNTED, MOUNT_POINT}
    """
    # Create dict with relevant infos
    partitions_dict = {}
    for partition in raw_lsblk:
        if len(partition) < 3:
            # Ignore if no FSTYPE detected
            continue

        if len(partition) < 4:
            is_mounted = False
            mount_point = None
        else:
            is_mounted = True
            mount_point = partition[3]

        partitions_dict[partition[0]] = {
            "FSTYPE": partition[2],
            "IS_MOUNTED": is_mounted,
            "MOUNT_POINT": mount_point,
        }

    # Warn the user if no partition found with lsblk
    if not partitions_dict:
        LOGGER.write("Error", "No partition found !")
        window.show_error_popup("Hum...", "No partition found.")
        return None

    LOGGER.write("debug", "Partition list generated using 'lsblk'")
    LOGGER.write("debug", f"{len(partitions_dict)} partitions found")

    return partitions_dict


def is_installed(command: str) -> bool:
    """Verify if 'progress' tool is installed on current system.

    Args:
        command (str): Command to be queried.

    Returns:
        bool: 'progress' is installed.
    """
    output = call(["which", command])

    return output == 0
